/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.features

import dev.inmo.krontab.buildSchedule
import dev.inmo.krontab.doInfinity
import dev.kord.common.Color
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.UserBehavior
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.channel.Channel
import dev.kord.core.entity.channel.MessageChannel
import dev.kord.rest.builder.message.EmbedBuilder
import dev.kord.rest.builder.message.create.actionRow
import kotlinx.coroutines.delay
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.entry.PlanningNotifierRolesData
import net.moonleay.lilJudd.data.tables.PlanningNotifierRoles
import net.moonleay.lilJudd.data.tables.TimePlanningChannels
import net.moonleay.lilJudd.data.tables.TimePlanningMessages
import net.moonleay.lilJudd.extensions.FeatureManageExtension
import net.moonleay.lilJudd.features.component.FeatureEnum
import net.moonleay.lilJudd.features.component.IFeature
import net.moonleay.lilJudd.util.EmbedUtil
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import net.moonleay.lilJudd.util.TimeUtil
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.ZoneId
import java.time.ZonedDateTime


object TimeManager : IFeature {
    override val feat: FeatureEnum
        get() = FeatureEnum.TIMEPLANNINGFEATURE

    // Register the cronjob to run at 0:01 AM UTC every Monday
    override suspend fun registerThread() {
        Logger.out("Adding message scheduler...")
        val scheduler = buildSchedule("0 0 1 * * * 0o 1w") // 0 0 4 * * * 0o 1w // 0o is UTC
        scheduler.doInfinity {
            this.runThread()
        }
    }

    private suspend fun runThread() {
        Logger.out("Starting to notify...")


        // ChannelID, ServerID
        val channelList = mutableMapOf<Snowflake, Snowflake>()
        // ChannelID, Data
        val roleMap = mutableMapOf<Snowflake, PlanningNotifierRolesData>()

        var msgStr = ""

        transaction {
            for (tp in TimePlanningChannels.selectAll()) {
                channelList[Snowflake(tp[TimePlanningChannels.channelid])] =
                    Snowflake(tp[TimePlanningChannels.serverid])
                Logger.out("Have to notify channel with ID ${tp[TimePlanningChannels.channelid]}.")
            }

            for (pnr in PlanningNotifierRoles.selectAll()) {
                roleMap[Snowflake(pnr[PlanningNotifierRoles.channelid])] = PlanningNotifierRolesData(
                    pnr[PlanningNotifierRoles.serverid],
                    pnr[PlanningNotifierRoles.channelid],
                    pnr[PlanningNotifierRoles.hastimeroleid],
                    pnr[PlanningNotifierRoles.wantstobenotifiedid]
                )
                Logger.out("Have to ping roles: ${pnr[PlanningNotifierRoles.wantstobenotifiedid]}}")
            }
        }
        Logger.out("${channelList.count()} Channels to notify with ${roleMap.count()} Roles to ping!")
        for (ch in channelList.keys) {
            if (Bot.bot.kordRef.getChannel(ch) == null)
                continue // TODO: Check if the channel is valid in another shard
            val c = Bot.bot.kordRef.getChannelOf<MessageChannel>(ch)!!
            if (roleMap != null && roleMap.keys.contains(ch) && roleMap[ch] != null) {
                c.createMessage {
                    this.content =
                        "The weekly planning starts now <@&${Snowflake(roleMap[ch]?.wantstobenotifid!!)}>"
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0X4C4645),
                            "Time Planning Feature",
                            "Do you have time on the following Days?",
                            "Automated Message"
                        )
                    )
                }
            } else {
                c.createMessage {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0X4C4645),
                            "Time Planning Feature",
                            "Do you have time on the following Days?",
                            "Automated Message"
                        )
                    )
                }
            }
            delay(2000)
            var then = ZonedDateTime.now(ZoneId.of("Europe/Berlin")).withHour(4).withMinute(0).withSecond(0)
            repeat(7) {
                val msg = c.createMessage {
                    this.embeds.add(
                        MessageUtil.getEmbedWithTable(
                            Color(0X4C4645),
                            "",
                            "${then.dayOfWeek.name}, ${then.dayOfMonth}.${then.monthValue}.${then.year} /${it + 1}. weekday",
                            mapOf(
                                "Is available" to listOf(),
                                "May be available" to listOf(),
                                "Is not available" to listOf()
                            )
                        )
                    )

                    this.actionRow {
                        this.components.addAll(EmbedUtil.getTimePlannerButtons().components)
                    }
                }
                msgStr += "${it}:${msg.id.value};"
                then = then.plusDays(1).withHour(4).withMinute(0).withSecond(0)
                Logger.out("Finished sending day $it")

                delay(1000)
            }

            // Save the message ids
            transaction {
                TimePlanningMessages.insert {
                    it[TimePlanningMessages.serverid] = c.data.guildId.value.toString()
                    it[TimePlanningMessages.channelid] = c.id.value.toString()
                    it[TimePlanningMessages.weekstamp] = TimeUtil.getWeekStamp().toOffsetDateTime().toString()
                    it[TimePlanningMessages.messageids] = msgStr
                } get TimePlanningMessages.id
            }
            Logger.out("Finished with ${c.data.guildId.value}")
        }
        Logger.out("Done! Until next Monday! <3   ")
    }

    override suspend fun enable(
        u: UserBehavior,
        gID: String,
        cID: String,
        ch: Channel,
        args: FeatureManageExtension.FeatureManagerArgs
    ): EmbedBuilder {
        var alreadyExists = false
        transaction {
            alreadyExists = TimePlanningChannels.select {
                (TimePlanningChannels.serverid eq gID) and
                        (TimePlanningChannels.channelid eq cID)
            }.count() > 0
        }
        if (!alreadyExists) {
            transaction {
                TimePlanningChannels.insert {
                    it[TimePlanningChannels.serverid] = gID
                    it[TimePlanningChannels.channelid] = cID
                } get TimePlanningChannels.id
            }
            return MessageUtil.getEmbed(
                Color(0x52E01A),
                "200: Success",
                "The feature was enabled in channel ${args.channel.data.name.value}",
                u.asUser().username + "#" + u.asUser().discriminator
            )
        }
        return MessageUtil.getEmbed(
            Color(0xE0311A),
            "403: Forbidden",
            "The feature is already enabled in this channel.",
            u.asUser().username + "#" + u.asUser().discriminator
        )
    }

    override suspend fun disable(
        u: UserBehavior,
        gID: String,
        cID: String,
        ch: Channel,
        args: FeatureManageExtension.FeatureManagerArgs
    ): EmbedBuilder {
        // Check if entry exists in db
        var alreadyExists = false
        transaction {
            alreadyExists = TimePlanningChannels.select {
                (TimePlanningChannels.serverid eq gID) and
                        (TimePlanningChannels.channelid eq cID)
            }.count() > 0
        }
        if (alreadyExists) {
            // delete all entrys for this channel
            transaction {
                val matchingEntries = TimePlanningChannels.select {
                    (TimePlanningChannels.serverid eq gID) and
                            (TimePlanningChannels.channelid eq cID)
                }.toList()

                matchingEntries.forEach { entry ->
                    TimePlanningChannels.deleteWhere { id eq entry[id] }
                }
            }
            return MessageUtil.getEmbed(
                Color(0x52E019),
                "200: Success",
                "The feature was disabled.",
                u.asUser().username + "#" + u.asUser().discriminator
            )
        }
        // Do nothing; not in db
        return MessageUtil.getEmbed(
            Color(0xE0311A),
            "403: Forbidden",
            "The feature is already disabled in this channel.",
            u.asUser().username + "#" + u.asUser().discriminator
        )
    }
}
